;; (find-eev "")

;;;###autoload
(defun ee-goto-position (&optional pos-spec &rest rest)
  "Go to a certain position in the current buffer is POS-SPEC is not nil.
If POS-SPEC is a string, search for its first occurence, starting from
the beginning of the accessible area; if it is a number, go to that
line; if POS-SPEC if nil then do nothing and leave point where it is.\n
See the documentation for the functions `find-fline' and `find-node'."
  (when pos-spec
    (goto-char (point-min))
    (cond ((numberp pos-spec)
	   (forward-line (1- pos-spec)))
	  ((stringp pos-spec)
	   (search-forward pos-spec))
	  (t (error "This is not a valid pos-spec: %S" pos-spec)))
    (if rest (ee-goto-rest rest))))

(defun ee-goto-rest (list)
  "Process any extra arguments given to `ee-goto-position'.
You are encouraged to redefine this function to make it more powerful."
  (cond ((stringp (car list))
	 (search-forward (car list))
	 (ee-goto-rest (cdr list)))))

;;;###autoload
(defun find-fline (fname &rest pos-spec-list)
  "Visit a file, and optionally go to a certain position in it.
This function is often used as a hyperlink: if you place the point
at the end of a line like one of those below\n
  # (find-fline \"/etc/resolv.conf\")
  # (find-fline \"~root/\")
  # (find-fline \"$HOME/.emacs\" 25)\n
and type `\\[eval-last-sexp]' Emacs will open the corresponding file.\n
For more information on the format of POS-SPEC-LIST, see the
documentation for the function `ee-goto-position'; for ways to omit part of
the directory specification, see the documentation for the function
`code-c-d'."
  (find-file (substitute-in-file-name fname))
  (apply 'ee-goto-position pos-spec-list))

;;;###autoload
(defun find-fline-nosubst (fname &rest pos-spec-list)
  "Like `find-fline', but without running `substitute-in-file-name' on FNAME."
  (find-file fname)
  (apply 'ee-goto-position pos-spec-list))

(if (not (fboundp 'Info-goto-node))
    (autoload 'Info-goto-node "info"))

;;;###autoload
(defun find-node (nodestr &rest pos-spec-list)
  "Open an Info node, and optionally go to a certain position in it.
This function is often used as a hyperlink: if you place the point
at the end of a line like one of those below\n
  # (find-node \"(emacs-lisp-intro)Run a Program\")
  # (find-node \"(emacs-e20)Lisp Eval\" \"C-x C-e\")
  # (find-node \"(emacs)Lisp Eval\")\n
and type `\\[eval-last-sexp]' Emacs will open the corresponding Info node.\n
For more information on the format of POS-SPEC-LIST, see the
documentation for the function `ee-goto-position'; for ways to omit the
\"(info-file-name)\" part of NODESTR, see the documentation for the
function `code-c-d'."
  (if (Info-goto-node nodestr)
      (apply 'ee-goto-position pos-spec-list)))



;;;;;
;;
;; emacs-hyperlinks
;;
;;;;;

(defun find-efunction (symbol &rest rest)
  (let ((location (find-function-noselect symbol)))
    (switch-to-buffer (car location))
    (goto-char (cdr location))
    (ee-goto-rest rest)))

(defun find-evariable (symbol &rest rest)
  (let ((location (find-variable-noselect symbol)))
    (switch-to-buffer (car location))
    (goto-char (cdr location))
    (ee-goto-rest rest)))

;; (find-efunction 'view-lossage)
;; (find-efunction 'code-c-d)
;; (find-efunction 'find-efile)



;;;;;
;;
;; find-progoutput
;;
;;;;;

;; (find-elnode "Synchronous Processes" "Function: shell-command-to-string")
;; (find-elnode "Buffer Basics")
;; (find-elnode "Buffer Names" "Function: get-buffer")
;; (find-progoutput "wget --help" "--continue")
;; (find-progoutput "set")

(defun find-progoutput (command &rest pos-spec-list)
  (if (get-buffer command)		; if the buffer already exists
      (switch-to-buffer command)	; then just switch to it
    (switch-to-buffer command)		; otherwise create it
    (insert (shell-command-to-string command)) ; prepare its contents
    (goto-char (point-min)))		; and place point at its beginning
  (apply 'ee-goto-position pos-spec-list))



;;;;;
;;
;; dvi-and-ps
;;
;;;;;

(defun find-dvipage (fname n)
  (let ((command (format "xdvi +%d %s &" n fname)))
    (eev command nil)
    command))
(defun find-pspage (fname n)
  (let ((command (format "gv -page %d %s &" n fname)))
    (eev command nil)
    command))



;;;;;
;;
;; find-man
;;
;;;;;

;; (find-elnode "Scope")
;; (find-elnode "Sentinels")
;; (find-efile "man.el" "set-process-sentinel")
;; (find-efile "man.el" "defun Man-bgproc-sentinel")
;;
;; Sentinel functions run at top-level, so we need a global
;; pos-spec-list (binding it in find-man.el wouldn't do), and we call
;; it ee-pos-spec-list-for-find-man.el. Always after using
;; ee-pos-spec-list-for-find-man.el we set it back to nil to avoid
;; letting it interfere with invoking man the standard way.

(setq ee-pos-spec-list-for-find-man.el nil)

(defun find-man.el (manpage &rest pos-spec-list)
  (setq ee-pos-spec-list-for-find-man.el pos-spec-list)
  (man manpage))

(defun find-man (manpage &rest pos-spec-list)
  (if (null ee-arg) (apply 'find-man.el manpage pos-spec-list)
    (apply 'eeman manpage pos-spec-list)))

;; (find-man.el "1 more" "-num")



;;;;;
;;
;; debian-hyperlinks
;;
;;;;;

;; For info on Debian packages. See `dff' below.
;; Here we use another format for anchors.
;;
(defun find-Package (fname &optional packagename &rest rest)
  "An auxiliary function to find package descriptions in Debian \"Packages\" files."
  (find-fline fname)
  (if packagename
      (apply 'ee-goto-position (format "\nPackage: %s\n" packagename) rest)))

(defun find-status (packagename &rest rest)
  "A hyperlink to the information in /var/lib/dpkg/status about a Debian package.
See the documentation on `dff'."
  (apply 'find-Package "/var/lib/dpkg/status" packagename rest))

(defun find-available (packagename &rest rest)
  "A hyperlink to the information in /var/lib/dpkg/available about a Debian package.
See the documentation on `dff'."
  (apply 'find-Package "/var/lib/dpkg/available" packagename rest))



;;;;;
;;
;; anchors
;;
;;;;;

;;;; Find strings enclosed in angle quotation marks.
;;;; These functions are especially convenient for when you have to
;;;; convert your files with hyperlinks to html.
;;
(defun find-anchor (fname &optional tag &rest rest)
  "An auxiliary function to find \"anchor strings\" in text files."
  (find-fline fname)
  (if tag (apply 'ee-goto-position (format ee-anchor-format tag) rest)))



;;;;;
;;
;; hyperlinks-to-the-same-file
;;
;;;;;

;;;; Hyperlinks to anchors in the same buffer, with a "back" function.
;;;; The "ee" prefix is somewhat misleading: these functions are
;;;; related to following hyperlinks, not to saving blocks of text.
;;;;
;;
;; Note: the "back" list is currently buffer-local, and saves only line
;; numbers. Reason: this was easier to code.
;; (find-elnode "Intro to Buffer-Local" "`make-variable-buffer-local'")
;;
(make-variable-buffer-local 'ee-back-list)
(setq-default ee-back-list nil)		; for XEmacs

(defun ee-save-position ()
  "Push an `(ee-goto-position LINENOW)' at the head of `ee-back-list'."
  (setq ee-back-list
	(cons (list 'ee-goto-position (count-lines (point-min) (point)))
	      ee-back-list)))

(defun ee-to (anchor)
  (interactive "sAnchor: ")
  "Local hyperlink to ANCHOR, pushing the current line into `ee-back-list'."
  (ee-save-position)
  (ee-goto-position (format ee-anchor-format anchor)))

(defun ee-back ()
  "Pop `ee-back-list', going to the position stored in the popped element."
  (interactive)
  (eval (car ee-back-list))
  (setq ee-back-list (cdr ee-back-list)))



;; Local Variables:
;; coding:               no-conversion
;; ee-anchor-format:     "%s"
;; ee-charset-indicator: ""
;; End:
