#!/bin/awk -f
# URLs for this file:
# Version: 2004mar18; copyleft Edrx. License: GPL.
#
# changer.awk: an awk script that is used by changer.sh and
# "make dotfiles-install" to modify initialization files.
# (find-eevnew "change")
# (find-eev "Makefile" "dotfiles")
# (find-eev "INSTALL" "eevblock-file-format")

# If you have never seen an Awk program before, the following links
# should take you to the pages in the Gawk Manual that describe all
# the features used in this script.
#
# (find-node "(gawk)Executable Scripts" "#! /bin/awk -f")
# (find-node "(gawk)Running gawk" "awk -f PROGRAM-FILE INPUT-FILE1")
# (find-node "(gawk)Assignment Options" "-v VARIABLE=TEXT")
# (find-node "(gawk)Options" "`-v VAR=VAL'")
# (find-node "(gawk)Very Simple")
# (find-node "(gawk)Pattern Overview" "`BEGIN'")
# (find-node "(gawk)Using BEGIN/END")
# (find-node "(gawk)Getline/Variable/File" "`getline VAR < FILE'")
# (find-node "(gawk)Getline/File")
# (find-node "(gawk)Typing and Comparison" "`X == Y'")
# (find-node "(gawk)While Statement")
# (find-node "(gawk)Basic Printf" "printf FORMAT, ITEM1, ITEM2, ...")
# (find-node "(libc)Formatted Output Basics" "Processing of `%s' is %d%%")
# (find-node "(libc)Table of Output Conversions" "`%s'")
# (find-node "(gawk)Using Variables" "initialized to")
# (find-node "(gawk)Statements/Lines" "continue")
# (find-node "(gawk)Fields" "`$0'")
#
# (find-node "(gawk)Index")


# (find-node "(gawk)Definition Syntax" "function NAME(PARAMETER-LIST)")
# (find-node "(gawk)Redirection" "`print ITEMS > OUTPUT-FILE'")
# (find-node "(gawk)Special FD" "`/dev/stderr'")
# (find-node "(gawk)Exit Statement")
#
function myerror (str) {
  printf "%s\n", str > "/dev/stderr";
  exit 1;
}

# Global variables used in the program:
#   changerdata       - a filename, set with -v.
#   blockname         - a word (i.e, a string without blanks), set with -v.
#   beginchangerblock - a word, taken from the first line of changerdata.
#   endchangerblock   - a word, taken from the first line of changerdata.
#   changerblockname  - a word, taken from the first line of changerdata.
#   begineevblock     - a line (i.e., a string that may contain blanks).
#   endeevblock       - a line.
#   state             - 0, 1, or 2

# The processing of the changerdata file is done in three stages:
# 1) in the BEGIN block, read its first line, and set the word variables
#    `beginchangerblock', `endchangerblock', and `changeblockname'.
# 2) in block0(), skip everything until finding a line of the form
#    "<beginchangerblock> <blockname>".
# 3) in block1(), print everything that comes before a line of the form
#    "<endchangerblock> <blockname>". The last line before that is
#    stored in the variable endeevblock.
#
# The lines in the changerdata file coming after that are not read.
#
# The processing of stdin is also done in three stages:
# 1) state == 0: everything before the `begineevblock' line is printed.
# 2) state == 1: everything between `begineevblock' and `endeevblock'
#    is skipped, and in its place we print (by running block1()) the
#    replacement text taken from the changerdata file.
# 3) state == 2: everything after `endeevblock' is printed verbatim.
#    Further `begineevblock'/`endeevblock' lines are not treated
#    specially.
#
# If the input from stdin ends and we're still at state 0 -- that is,
# we haven't found a `begineevblock'/`endeevblock' block in stdin --
# then we output the block from changedata. 

# (eecd "gawk -f changer.awk < .BASHRC")



function block0 () {
  while (getline < changerdata) {
    if ($0 == (beginchangerblock " " blockname)) {
      getline begineevblock < changerdata;
      return;
    }
    if ($1 == changeblockname && blockname == $3) # a quick hack...
      blockname = $2;
  }
  myerror("The changerdata file doesn't have a \"" blockname "\" block")
}
function block1 () {
  print begineevblock;
  while (getline < changerdata) {
    if ($0 == (endchangerblock " " blockname))
      return;
    else {
      print;
      endeevblock = $0;		# holds the line before the endchangerblock
    }
  }
  myerror("The changerdata file doesn't close the \"" blockname "\" block")
}



BEGIN {
  getline < changerdata;
  if ($1 != "keywords:" || NF != 4)
    myerror("The changerdata file must begin with a \"delimiters\" line");
  beginchangerblock = $2;
  endchangerblock   = $3;
  changeblockname   = $4;
  block0();
  state = 0;
}

state == 0 {
  if ($0 == begineevblock) {
    block1();
    state = 1;
  }
  else
    print;
}

state == 1 {
  if ($0 == endeevblock) {
    state = 2;
    next;
  }
}

state == 2 {			
  print				
}				

END {				# on end of input,
  if (state == 0) {		# if we have never met a beg_eev_block,
    print "";			# print a blank line, as a separator,
    block1();
  }
  if (state == 1) {
    myerror("Input ended in the middle of the eev block")
  }
  if (state == 2) {
  }
}
