;;;;;
;;
;; From rubikitch@ruby-lang.org  Tue Jul 15 01:23:54 2003
;; Date: Tue, 15 Jul 2003 13:21:38 +0900 (JST)
;; From: rubikitch <rubikitch@ruby-lang.org>
;; To: edrx@mat.puc-rio.br
;; Subject: eev is wonderful!!
;;
;;;;;

;; Hi, I'm rubikitch in Japan.
;; Eev is wonderful!! I love it!!
;; So my personal diary(of course e-scripts) includes a lot of sexps.
;; 
;; In my .emacs, there are a lot of eev-related codes.
;; For example:

(defun ee-bookmark (fn pos front rear)
  (cond ((bufferp fn)
         (switch-to-buffer fn))
        ((stringp fn)
         (find-file fn))
        (t
         (push-mark)))
  (goto-char pos)
  (if (and front (search-forward front (point-max) t))
      (goto-char (match-beginning 0)))
  (if (and rear (search-backward rear (point-min) t))
      (goto-char (match-end 0))))

(defalias 'pos 'ee-bookmark)
(setq ee-bookmark-tmp nil)

(defun ee-yank (arg)
  (interactive "P")
  (when ee-bookmark-tmp
    (let ((print-escape-newlines t)
          (buf (car ee-bookmark-tmp))
          (fn (nth 1 ee-bookmark-tmp))
          (pos (nth 2 ee-bookmark-tmp))
          (front (nth 3 ee-bookmark-tmp))
          (rear (nth 4 ee-bookmark-tmp)))
      (setq ee-bookmark-tmp nil)
      (kill-new (if (eq buf (current-buffer))
                    (format "(pos nil %d %S %S)" pos front rear)
                  (format "(pos %S %d %S %S)" fn pos front rear)))))
  (call-interactively 'yank))
(global-set-key "\C-y" 'ee-yank)

(defun ee-make-bookmark (arg)
  (interactive "P")
  (require 'bookmark)
  (let* ((bookmark "ee")
         (bookmark-alist (list (list bookmark (bookmark-make-cell nil)))) 
         (filename (unless arg (bookmark-get-filename bookmark)))
         (front (bookmark-get-front-context-string bookmark))
         (rear (bookmark-get-rear-context-string bookmark))
         (position (bookmark-get-position bookmark))
         )
    (setq ee-bookmark-tmp (list (current-buffer) filename position front rear))
    (message "stored position at #<buffer %s> to kill-ring."
	     (buffer-name (current-buffer)))))

;; ee-make-bookmark saves the buffer-file-name and the position of the
;; buffer in emacs-lisp format, then you can yank it. My diary
;; contains many pos functions.
;; 
;; e.g.

;; (pos "/home/rubikitch/emacs/init.d/50eev.el" 6330 "(define-key nfer" "-make-bookmark)\n")




;; And I extended ee-goto-position so that its argument accepts lists.
;; The list must be (point front rear).

(defun ee-goto-position (&optional pos-spec &rest rest)
  "Go to a certain position in the current buffer is POS-SPEC is not nil.
If POS-SPEC is a string, search for its first occurence, starting from
the beginning of the accessible area; if it is a number, go to that
line; if POS-SPEC if nil then do nothing and leave point where it is.\n
See the documentation for the functions `find-fline' and `find-node'."
  (when pos-spec
    (goto-char (point-min))
    (cond ((numberp pos-spec)
	   (forward-line (1- pos-spec)))
	  ((stringp pos-spec)
	   (search-forward pos-spec))
          ((listp pos-spec)
           (let ((pos (car pos-spec))
                 (front (nth 1 pos-spec))
                 (rear (nth 2 pos-spec)))
             (goto-char pos)
             (if (and front (search-forward front (point-max) t))
                 (goto-char (match-beginning 0)))
             (if (and rear (search-backward rear (point-min) t))
                 (goto-char (match-end 0)))))
	  (t (error "This is not a valid pos-spec: %S" pos-spec)))
    (if rest (ee-goto-rest rest))))

;; Applying this extension, the pos function can rewrite the following.
;; 
;; (pos "/home/rubikitch/emacs/init.d/50eev.el" 6330 "(define-key nfer" "-make-bookmark)\n")
;; (find-fline "/home/rubikitch/emacs/init.d/50eev.el" '(6330 "(define-key nfer" "-make-bookmark)\n"))
;; 
;; Of course, you can use this extension with other hyperlink function
;; generated by code-c-d.




;; I also make sexp-navigating commands.

(defun ee-next-sexp ()
  (interactive)
  (unless (re-search-forward ")$" nil t)
    (error "sexp not found")))

(defun ee-prev-sexp ()
  (interactive)
  (when (eq (char-before) ?\))
    (forward-char -1))
  (if (re-search-backward ")$" nil t)
      (forward-char 1)
    (error "sexp not found")))

(global-set-key "\M-p" 'ee-prev-sexp)
(global-set-key "\M-n" 'ee-next-sexp)




;; And enhanced `back' feature. You can back the window-configuretion
;; before eval the sexp.

(setq ee-winconf-stack nil)
(defun ee-push-winconf ()
  (setq ee-winconf-stack (cons (current-window-configuration)
			       ee-winconf-stack)))
(defun ee-end-of-line-then-eval-last-sexp (ee-arg)
  (interactive "P")
  (require 'bookmark)
  (let* (;(display-buffer-function 'switch-to-buffer)
         (display-buffer-function nil)
         ;(temp-buffer-show-function 'switch-to-buffer)
         (temp-buffer-show-function nil))
    (ee-push-winconf)
    (progn
      (end-of-line-then-eval-last-sexp ee-arg))))
(defun ee-back-2 ()
  (interactive)
  (let ((arg (car ee-winconf-stack)))
    (setq ee-winconf-stack (cdr ee-winconf-stack))
    (if arg
        (set-window-configuration arg))))
(define-key esc-map "e" 'ee-end-of-line-then-eval-last-sexp)




;; The link functions generated by code-c-d is a bit difficult to
;; write, I thought. Because there's no input-helping function except
;; ee-insert-info-hyperlink. I cannot write such a function without
;; completion feature. So I'll try to write input-helping function.

;; I am a Ruby addict. I write many Ruby scripts. You also use Ruby?
;; I made a eev function to jump to RAA(Ruby Application Archive).
;; It's `find-raa'. It needs rnet library.
;; Sorry for only Japanese document.... But you may use it easily.
;; http://www.ruby-lang.org/~rubikitch/archive/find-raa-0.0.tar.gz
;; http://www.ruby-lang.org/~rubikitch/archive/rnet-0.1.1.tar.gz

;; I use emacs21(-nw) and GNU Screen and w3m(yet another text browser).
;; http://w3m.sourceforge.net/
;; And emacs-w3m is much faster than W3.
;; http://emacs-w3m.namazu.org/index-en.html
;; 
;; rubikitch


